==> stam_1.4.0.tar.gz: stam/R/02_node.R <==
################################################################################
#
#   02_node.R
#
#   Structural information on a node including GO-IDs, indices
#   of children and indices of genes. Moreover, results from
#   fitting are stored (weights for genes and child-terms).
#
#   written by Claudio Lottaz (COMPDIAG-MPI-MG), in August 2004
#
################################################################################
#
#   Class Definition and Constructor
#

setClass("stamNode", 
         representation(ID              = "character", # GO idnentifier
                        category        = "character", # which ontology (BP, CC, MF)
                        replacedParents = "character", # one-child-parents removed
                        supNode         = "logical"))  # TRUE for Leaf with same-ID-INode

setMethod("initialize", "stamNode",
  function(.Object, ID, supNode=FALSE) {
    if (missing(ID)) stop("No ID for GO-node provided\n")
    if (is.null(getGOterm(ID))) stop("GO-node ", ID, " not found")
    .Object@ID <- ID
    .Object@category <- getGOcategory(ID)
    .Object@replacedParents <- character(0)
    .Object@supNode <- supNode
    return(.Object)
  }
)

setClass("stamLeaf",
         representation(chip   = "character",  # chip the probes correspond
                        probes = "numeric",    # idx according to order in expr.mat
                        pamfit = "pamrtrained",# fit for probes only
                        delta  = "numeric"),   # delta fr lowest error-rate in above fit
         contains = "stamNode")

setMethod("initialize", "stamLeaf",
  function(.Object, ID, chip="hgu95av2", probesidx=NULL) {
    .Object <- callNextMethod(.Object, ID=ID)
    .Object@chip <- chip
    probeids <- getGOprobes(ID, chip)
    if (is.null(probesidx)) probesidx <- getprobes(chip)
    .Object@probes <- stam.getidx(probeids, probesidx)
    emptyfit <- list()
    class(emptyfit) <- "pamrtrained"
    .Object@pamfit <- emptyfit
    .Object@delta <- 0
    return(.Object)
  }
)

setClass("stamINode",
         representation(children = "numeric",     # postorder index
                        weights  = "numeric"),    # weight according the same order
         contains="stamNode")

setMethod("initialize", "stamINode",
  function(.Object, ID, GOidx=NULL, children=NULL) {
    if (is.null(GOidx) && is.null(children)) 
      stop("Neither child indices nor GO-index environment provided")
    if (!is.null(GOidx) && !is.null(children)) 
      stop("Both child indices and GO-index environemt provided")
    .Object <- callNextMethod(.Object, ID=ID)
    if (is.null(GOidx)) { .Object@children <- children 
    } else .Object@children <- stam.getidx(getGOchildren(.Object), GOidx)
    nchildren <- length(.Object@children)
    if (nchildren > 0) {
      .Object@weights <- rep(1/nchildren, times=nchildren)
      names(.Object@weights) <- names(children)
    } else .Object@weights <- numeric(0)
    return(.Object)
  }
)


################################################################################
#
#   Print and Summary
#

setMethod("print", "stamNode",
  function(x,...) {
    cat(class(x), x@ID, getGOterm(x), "\n")
    if (x@category == "GO") gocat <- "gene ontology"
    if (x@category == "BP") gocat <- "biological process"
    if (x@category == "CC") gocat <- "cell components"
    if (x@category == "MF") gocat <- "molecular function"
    cat("from category", gocat, "\n")
    if (length(x@replacedParents) > 0) {
      if (length(x@replacedParents) > 1) plural <- "s" else plural <- ""
      cat(length(x@replacedParents), " parent node", plural, " replaced:\n", sep="")
      for (id in x@replacedParents) cat(id, ":", getGOterm(id), "\n")
    }
  }
)

setMethod("print", "stamLeaf",
  function(x, exprs, cv, ...) {
    callNextMethod()
    ngenes <- length(x@probes)
    if (ngenes == 0) cat("contains no genes\n")     
    if (ngenes == 1) cat("contains one gene\n")     
    if (ngenes >= 2) cat("contains", ngenes, "genes\n")     
    if (length(x@pamfit) == 0) print(ls(getprobes(x@chip))[x@probes])
    else {
      if (missing(exprs) && (!exists(".stam.exprs", .GlobalEnv))) return(invisible(NULL))
      if (!missing(exprs)) .stam.exprs <- exprs
      if (missing(cv) && (!exists(".stam.cv", .GlobalEnv))) return(invisible(NULL))
      if (!missing(cv)) .stam.cv <- cv
      fit <- reconstruct.pamfit(x@pamfit, .stam.cv)
      res <- stam.listgenes(fit, x@delta, x@chip, .stam.exprs, .stam.cv@sample.classes)
      if (is.null(res)) { cat("None of the genes allows for classification\n")
      } else {
        cat("The following ", nrow(res), " of these are used in classification:\n")
        print(res, quote=FALSE)
      }
    }
  }
)

setMethod("print", "stamINode",
  function(x,...) {
    callNextMethod()
    nchildren <- length(x@children)
    if (nchildren == 0) cat("contains no children\n")
    if (nchildren == 1) cat("contains 1 child\n")
    if (nchildren >= 2) 
      cat("contains", nchildren, "children,", sum(x@weights>0), "of which have weights\n")
    if (length(x@weights) > 0) print(x@weights)
  }
)


################################################################################
#
#   HTML and XML
#

nodelink <- function(id, sepfiles=FALSE) {
  if (!sepfiles) return(paste("<A HREF='#",id,"'>",id,"</A>", sep=""))
  tmp <- gsub(":", "", id)
  return(paste("<A HREF='",tmp,".html' TARGET='NodeInfo'>",id,"</A>", sep=""))
}
 
setMethod("writeHTML", "stamNode",
  function(x, sepfiles=FALSE, ...) {
    if (x@category =="GO root") 
      gocat <- "<TD width=100 bgcolor='lightgrey'>Gene<BR>Ontology</TD></TR></TABLE></P>"
    if (x@category =="BP") 
      gocat <- "<TD width=100 bgcolor='yellow'>Biological<BR>Process</TD></TR></TABLE></P>"
    if (x@category =="CC") 
      gocat <- "<TD width=100 bgcolor='lightgreen'>Cellular<BR>Component</TD></TR></TABLE></P>"
    if (x@category =="MF") 
      gocat <- "<TD width=100 bgcolor='lightblue'>Molecular<BR>Function</TD></TR></TABLE></P>"
    a <- ifelse(x@supNode, "a", "")
    catHTML(HR(), NAME(paste(x@ID, a, sep="")))
    if (class(x) == "stamLeaf") {
      a <- "<BR><FONT size=\"-2\">(annotated genes)</FONT>"
      nodecolor = "#DDDDDD"
    } else {
      a <- ""
      nodecolor = "#BBBBBB"
    }
    for (id in x@replacedParents) catHTML(NAME(id))
    term <- getGOterm(x)
    catHTML(paste("<P><TABLE border='3' width='100%'><TR valign='middle'><TD bgcolor='", nodecolor,
            "' width=150><H2>", GOREF(x@ID), "</H2></TD><TD><H2>", term, a, "</H2></TD>", gocat))
    nreplaced <- length(x@replacedParents)
    if (nreplaced > 0) {
      catHTML("<H3>", nreplaced)
      if (nreplaced > 1) { catHTML(" parent nodes replaced:</H3><P><UL><TABLE>")
      } else catHTML(" parent node replaced:</H3><P><UL><TABLE>")
      for (id in x@replacedParents) 
        catHTML("<TR valign='top'><TH align='right' bgcolor='lightgreen'>", id, 
                "</TD><TD>", getGOterm(id), "</TD></TR>")
      catHTML("</TABLE></UL></P>")
    }
    if (x@supNode) { # supNodes (leafs) have their corresponding INode as only parent
      catHTML("<H3>Parent node</H3><P><UL><TABLE><TR valign='top'><TH align='right' bgcolor='lightgreen'>", nodelink(x@ID, sepfiles), "</TH><TD>", term, "</TD></TR></TABLE></UL></P>")
    } else {
      parents <- getGOparents(x)
      for (id in x@replacedParents) parents <- unique(c(parents, getGOparents(id)))
      parents <- setdiff(parents, x@replacedParents)
      if (length(parents) > 0) {
        if (length(parents) > 1) { catHTML("<H3>Parent nodes</H3><P><UL><TABLE>")
        } else catHTML("<H3>Parent node</H3><P><UL><TABLE>")
        for (id in parents) {
          if (exists(id, .stam.GOids)) {
            catHTML("<TR valign='top'><TH align='right' bgcolor='lightgreen'>",
                    nodelink(id, sepfiles), "</TH><TD>", getGOterm(id), "</TD></TR>")
          } else {
            catHTML("<TR valign='top'><TH align='right' bgcolor='lightgreen'>",
                    id, "</TH><TD>", getGOterm(id), " <I>(not in fit)</I></TD></TR>")
          }
        }
        catHTML("</TABLE></UL></P>")
      }
    }
  }
)

setMethod("writeHTML", "stamLeaf",
  function(x, exprs, cv, sepfiles, ...) {
    callNextMethod(x, sepfiles=sepfiles)
    ngenes <- length(x@probes)
    if (ngenes == 0) catHTML("<H3>This leaf node contains no genes</H3>")
    if (ngenes == 1) catHTML("<H3>This leaf node contains one gene</H3>")
    if (ngenes >= 2) catHTML("<H3>This leaf node contains ", ngenes, " genes</H3>")

    if (length(x@pamfit) == 0) {
      # .stam.allaffyids may exist in the global environment to speed up things
      if (exists(".stam.allaffyids")) { affyids <- .stam.allaffyids[x@probes]
      } else affyids <- ls(getprobes(x@chip))[x@probes]
      catHTML("<P><UL>")
      for (id in affyids) catHTML(AFFYREF(id), " ")
      catHTML("</UL></P>")
    } else {
      if (missing(exprs) && (!exists(".stam.exprs", .GlobalEnv))) return(invisible(NULL))
      if (!missing(exprs)) .stam.exprs <- exprs
      if (missing(cv) && (!exists(".stam.cv", .GlobalEnv))) return(invisible(NULL))
      if (!missing(cv)) .stam.cv <- cv
      fit <- reconstruct.pamfit(x@pamfit, .stam.cv)
      res <- stam.listgenes(fit, x@delta, x@chip, .stam.exprs, .stam.cv@sample.classes)
      if (is.null(res)) { catHTML("<P>None of the genes allows for classification</P>")
      } else {
        catHTML("<P>The following ", nrow(res), " of these are used in classification:</P>")
        rownames(res) <- unlist(lapply(rownames(res), AFFYREF))
        catHTML("<P>", tableHTML(res, cell.valign="top", header.bgcolor="lightblue", 
                                     header.valign="top"), "</P>")
      }
    }
  }
)

setMethod("writeHTML", "stamINode",
  function(x, sepfiles=FALSE, ...) {
    callNextMethod(x, sepfiles=sepfiles)

    if (exists(".stam.allnodes")) {
      catHTML("<H3>", length(x@children), 
              " child nodes</H3><P><UL><TABLE><TR><TH bgcolor='lightgreen'>ID</TH><TH bgcolor='lightgreen'>Term</TH><TH bgcolor='lightgreen'>Weight</TH></TR>")
      children <- names(x@children) #getGOchildren(x)
      for (c in 1:length(x@children)) {
        if (x@children[c] != 0) {
          id <- .stam.allnodes[[x@children[c]]]@ID
          if (.stam.allnodes[[x@children[c]]]@supNode) {
            htmlid <- nodelink(paste(id, "a", sep=""), sepfiles)
            catHTML("<TR valign='top'><TH align='right' bgcolor='lightgreen'>", htmlid, 
                    "</TH><TD>", getGOterm(id), " <I>(annotated genes)</I></TD><TD>",
                    x@weights[c], "</TD></TR>")
          } else {
            htmlid <- nodelink(id, sepfiles)
            catHTML("<TR valign='top'><TH align='right' bgcolor='lightgreen'>", htmlid,
                    "</TH><TD>", getGOterm(id), "</TD><TD>", round(x@weights[c], 5), "</TD></TR>")
          }
        } else { # child without weight
          id <- children[c]
          catHTML("<TR valign='top'><TH align='right' bgcolor='lightgreen'>", id,
                  "</TH><TD>", getGOterm(id), "</TD><TD>0.00000</TD></TR>")
        }

      }
      catHTML("</TABLE></UL></P>")
    } else {
      children <- getGOchildren(x)
      if (length(children) > 0) {
        catHTML(H3(length(children), " child nodes"), "<P><UL><TABLE>")
        for (id in children) 
          catHTML(TR(valign="top", TH(align="right", bgcolor="lightgreen", 
                                      HREF(id, url=paste("#", id, sep=""))),
                                   TD(getGOterm(id))))
        catHTML("</TABLE></UL></P>")
      }    
    }
  }
)


#
#   End of File
#
################################################################################

==> stam_1.8.0.tar.gz: stam/R/02_node.R <==
################################################################################
#
#   02_node.R
#
#   Structural information on a node including GO-IDs, indices
#   of children and indices of genes. Moreover, results from
#   fitting are stored (weights for genes and child-terms).
#
#   written by Claudio Lottaz (COMPDIAG-MPI-MG), in August 2004
#
################################################################################
#
#   Class Definition and Constructor
#

setClass("stamNode", 
         representation(ID              = "character", # GO idnentifier
                        category        = "character", # which ontology (BP, CC, MF)
                        replacedParents = "character", # one-child-parents removed
                        supNode         = "logical"))  # TRUE for Leaf with same-ID-INode

setMethod("initialize", "stamNode",
  function(.Object, ID, supNode=FALSE) {
    if (missing(ID)) stop("No ID for GO-node provided\n")
    if (is.null(getGOterm(ID))) stop("GO-node ", ID, " not found")
    .Object@ID <- ID
    .Object@category <- getGOcategory(ID)
    .Object@replacedParents <- character(0)
    .Object@supNode <- supNode
    return(.Object)
  }
)

setClass("stamLeaf",
         representation(chip   = "character",  # chip the probes correspond
                        probes = "numeric",    # idx according to order in expr.mat
                        pamfit = "pamrtrained",# fit for probes only
                        delta  = "numeric"),   # delta fr lowest error-rate in above fit
         contains = "stamNode")

setMethod("initialize", "stamLeaf",
  function(.Object, ID, chip="hgu95av2", probesidx=NULL) {
    .Object <- callNextMethod(.Object, ID=ID)
    .Object@chip <- chip
    probeids <- getGOprobes(ID, chip)
    if (is.null(probesidx)) probesidx <- getprobes(chip)
    .Object@probes <- stam.getidx(probeids, probesidx)
    emptyfit <- list()
    class(emptyfit) <- "pamrtrained"
    .Object@pamfit <- emptyfit
    .Object@delta <- 0
    return(.Object)
  }
)

setClass("stamINode",
         representation(children = "numeric",     # postorder index
                        weights  = "numeric"),    # weight according the same order
         contains="stamNode")

setMethod("initialize", "stamINode",
  function(.Object, ID, GOidx=NULL, children=NULL) {
    if (is.null(GOidx) && is.null(children)) 
      stop("Neither child indices nor GO-index environment provided")
    if (!is.null(GOidx) && !is.null(children)) 
      stop("Both child indices and GO-index environemt provided")
    .Object <- callNextMethod(.Object, ID=ID)
    if (is.null(GOidx)) { .Object@children <- children 
    } else .Object@children <- stam.getidx(getGOchildren(.Object), GOidx)
    nchildren <- length(.Object@children)
    if (nchildren > 0) {
      .Object@weights <- rep(1/nchildren, times=nchildren)
      names(.Object@weights) <- names(children)
    } else .Object@weights <- numeric(0)
    return(.Object)
  }
)


################################################################################
#
#   Print and Summary
#

setMethod("print", "stamNode",
  function(x,...) {
    cat(class(x), x@ID, getGOterm(x), "\n")
    if (x@category == "GO") gocat <- "gene ontology"
    if (x@category == "BP") gocat <- "biological process"
    if (x@category == "CC") gocat <- "cell components"
    if (x@category == "MF") gocat <- "molecular function"
    cat("from category", gocat, "\n")
    if (length(x@replacedParents) > 0) {
      if (length(x@replacedParents) > 1) plural <- "s" else plural <- ""
      cat(length(x@replacedParents), " parent node", plural, " replaced:\n", sep="")
      for (id in x@replacedParents) cat(id, ":", getGOterm(id), "\n")
    }
  }
)

setMethod("print", "stamLeaf",
  function(x, exprs, cv, ...) {
    callNextMethod()
    ngenes <- length(x@probes)
    if (ngenes == 0) cat("contains no genes\n")     
    if (ngenes == 1) cat("contains one gene\n")     
    if (ngenes >= 2) cat("contains", ngenes, "genes\n")     
    if (length(x@pamfit) == 0) print(ls(getprobes(x@chip))[x@probes])
    else {
      if (missing(exprs) && (!exists("exprs", stam.env))) return(invisible(NULL))
      if (!missing(exprs)) stam.env[["exprs"]] <- exprs
      if (missing(cv) && (!exists("cv", stam.env))) return(invisible(NULL))
      if (!missing(cv)) stam.env[["cv"]] <- cv
      fit <- reconstruct.pamfit(x@pamfit, stam.env[["cv"]])
      res <- stam.listgenes(fit, x@delta, x@chip, stam.env[["exprs"]], stam.env[["cv"]]@sample.classes)
      if (is.null(res)) { cat("None of the genes allows for classification\n")
      } else {
        cat("The following ", nrow(res), " of these are used in classification:\n")
        print(res, quote=FALSE)
      }
    }
  }
)

setMethod("print", "stamINode",
  function(x,...) {
    callNextMethod()
    nchildren <- length(x@children)
    if (nchildren == 0) cat("contains no children\n")
    if (nchildren == 1) cat("contains 1 child\n")
    if (nchildren >= 2) 
      cat("contains", nchildren, "children,", sum(x@weights>0), "of which have weights\n")
    if (length(x@weights) > 0) print(x@weights)
  }
)


################################################################################
#
#   HTML and XML
#

nodelink <- function(id, sepfiles=FALSE) {
  if (!sepfiles) return(paste("<A HREF='#",id,"'>",id,"</A>", sep=""))
  tmp <- gsub(":", "", id)
  return(paste("<A HREF='",tmp,".html' TARGET='NodeInfo'>",id,"</A>", sep=""))
}
 
setMethod("writeHTML", "stamNode",
  function(x, sepfiles=FALSE, ...) {
    if (x@category =="GO root") 
      gocat <- "<TD width=100 bgcolor='lightgrey'>Gene<BR>Ontology</TD></TR></TABLE></P>"
    if (x@category =="BP") 
      gocat <- "<TD width=100 bgcolor='yellow'>Biological<BR>Process</TD></TR></TABLE></P>"
    if (x@category =="CC") 
      gocat <- "<TD width=100 bgcolor='lightgreen'>Cellular<BR>Component</TD></TR></TABLE></P>"
    if (x@category =="MF") 
      gocat <- "<TD width=100 bgcolor='lightblue'>Molecular<BR>Function</TD></TR></TABLE></P>"
    a <- ifelse(x@supNode, "a", "")
    catHTML(HR(), NAME(paste(x@ID, a, sep="")))
    if (class(x) == "stamLeaf") {
      a <- "<BR><FONT size=\"-2\">(annotated genes)</FONT>"
      nodecolor = "#DDDDDD"
    } else {
      a <- ""
      nodecolor = "#BBBBBB"
    }
    for (id in x@replacedParents) catHTML(NAME(id))
    term <- getGOterm(x)
    catHTML(paste("<P><TABLE border='3' width='100%'><TR valign='middle'><TD bgcolor='", nodecolor,
            "' width=150><H2>", GOREF(x@ID), "</H2></TD><TD><H2>", term, a, "</H2></TD>", gocat))
    nreplaced <- length(x@replacedParents)
    if (nreplaced > 0) {
      catHTML("<H3>", nreplaced)
      if (nreplaced > 1) { catHTML(" parent nodes replaced:</H3><P><UL><TABLE>")
      } else catHTML(" parent node replaced:</H3><P><UL><TABLE>")
      for (id in x@replacedParents) 
        catHTML("<TR valign='top'><TH align='right' bgcolor='lightgreen'>", id, 
                "</TD><TD>", getGOterm(id), "</TD></TR>")
      catHTML("</TABLE></UL></P>")
    }
    if (x@supNode) { # supNodes (leafs) have their corresponding INode as only parent
      catHTML("<H3>Parent node</H3><P><UL><TABLE><TR valign='top'><TH align='right' bgcolor='lightgreen'>", nodelink(x@ID, sepfiles), "</TH><TD>", term, "</TD></TR></TABLE></UL></P>")
    } else {
      parents <- getGOparents(x)
      for (id in x@replacedParents) parents <- unique(c(parents, getGOparents(id)))
      parents <- setdiff(parents, x@replacedParents)
      if (length(parents) > 0) {
        if (length(parents) > 1) { catHTML("<H3>Parent nodes</H3><P><UL><TABLE>")
        } else catHTML("<H3>Parent node</H3><P><UL><TABLE>")
        for (id in parents) {
          if (exists(id, stam.env[["GOids"]])) {
            catHTML("<TR valign='top'><TH align='right' bgcolor='lightgreen'>",
                    nodelink(id, sepfiles), "</TH><TD>", getGOterm(id), "</TD></TR>")
          } else {
            catHTML("<TR valign='top'><TH align='right' bgcolor='lightgreen'>",
                    id, "</TH><TD>", getGOterm(id), " <I>(not in fit)</I></TD></TR>")
          }
        }
        catHTML("</TABLE></UL></P>")
      }
    }
  }
)

setMethod("writeHTML", "stamLeaf",
  function(x, exprs, cv, sepfiles, ...) {
    callNextMethod(x, sepfiles=sepfiles)
    ngenes <- length(x@probes)
    if (ngenes == 0) catHTML("<H3>This leaf node contains no genes</H3>")
    if (ngenes == 1) catHTML("<H3>This leaf node contains one gene</H3>")
    if (ngenes >= 2) catHTML("<H3>This leaf node contains ", ngenes, " genes</H3>")

    if (length(x@pamfit) == 0) {
      # stam.env[["allaffyids"]] may exist to speed up things
      if (exists("allaffyids", stam.env)) { affyids <- stam.env[["allaffyids"]][x@probes]
      } else affyids <- ls(getprobes(x@chip))[x@probes]
      catHTML("<P><UL>")
      for (id in affyids) catHTML(AFFYREF(id), " ")
      catHTML("</UL></P>")
    } else {
      if (missing(exprs) && (!exists("exprs", stam.env))) return(invisible(NULL))
      if (!missing(exprs)) stam.env[["exprs"]] <- exprs
      if (missing(cv) && (!exists("cv", stam.env))) return(invisible(NULL))
      if (!missing(cv)) stam.env[["cv"]] <- cv
      fit <- reconstruct.pamfit(x@pamfit, stam.env[["cv"]])
      res <- stam.listgenes(fit, x@delta, x@chip, stam.env[["exprs"]], stam.env[["cv"]]@sample.classes)
      if (is.null(res)) { catHTML("<P>None of the genes allows for classification</P>")
      } else {
        catHTML("<P>The following ", nrow(res), " of these are used in classification:</P>")
        rownames(res) <- unlist(lapply(rownames(res), AFFYREF))
        catHTML("<P>", tableHTML(res, cell.valign="top", header.bgcolor="lightblue", 
                                     header.valign="top"), "</P>")
      }
    }
  }
)

setMethod("writeHTML", "stamINode",
  function(x, sepfiles=FALSE, ...) {
    callNextMethod(x, sepfiles=sepfiles)

    if (exists("allnodes", stam.env)) {
      catHTML("<H3>", length(x@children), 
              " child nodes</H3><P><UL><TABLE><TR><TH bgcolor='lightgreen'>ID</TH><TH bgcolor='lightgreen'>Term</TH><TH bgcolor='lightgreen'>Weight</TH></TR>")
      children <- names(x@children) #getGOchildren(x)
      for (c in 1:length(x@children)) {
        if (x@children[c] != 0) {
          id <- stam.env[["allnodes"]][[x@children[c]]]@ID
          if (stam.env[["allnodes"]][[x@children[c]]]@supNode) {
            htmlid <- nodelink(paste(id, "a", sep=""), sepfiles)
            catHTML("<TR valign='top'><TH align='right' bgcolor='lightgreen'>", htmlid, 
                    "</TH><TD>", getGOterm(id), " <I>(annotated genes)</I></TD><TD>",
                    x@weights[c], "</TD></TR>")
          } else {
            htmlid <- nodelink(id, sepfiles)
            catHTML("<TR valign='top'><TH align='right' bgcolor='lightgreen'>", htmlid,
                    "</TH><TD>", getGOterm(id), "</TD><TD>", round(x@weights[c], 5), "</TD></TR>")
          }
        } else { # child without weight
          id <- children[c]
          catHTML("<TR valign='top'><TH align='right' bgcolor='lightgreen'>", id,
                  "</TH><TD>", getGOterm(id), "</TD><TD>0.00000</TD></TR>")
        }

      }
      catHTML("</TABLE></UL></P>")
    } else {
      children <- getGOchildren(x)
      if (length(children) > 0) {
        catHTML(H3(length(children), " child nodes"), "<P><UL><TABLE>")
        for (id in children) 
          catHTML(TR(valign="top", TH(align="right", bgcolor="lightgreen", 
                                      HREF(id, url=paste("#", id, sep=""))),
                                   TD(getGOterm(id))))
        catHTML("</TABLE></UL></P>")
      }    
    }
  }
)


#
#   End of File
#
################################################################################
